/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @flow
 */

/* eslint-env node */
import * as React from 'react';
import Stream from 'stream';
// $FlowFixMe - renderToPipeableStream
import {renderToString, renderToPipeableStream} from 'react-dom/server';
import type {Logger} from 'fusion-tokens';

// TODO: COPIED FROM fusion-core/src/plugins/ssr
function getUrls({chunkUrlMap, webpackPublicPath}, chunks) {
  // cross origin is needed to get meaningful errors in window.onerror
  const isCrossOrigin = webpackPublicPath.startsWith('http');
  const crossOriginAttribute = isCrossOrigin ? ' crossorigin="anonymous"' : '';
  return [...new Set(chunks)].map((id) => {
    const url = chunkUrlMap.get(id).get('es5');
    return {url, crossOriginAttribute};
  });
}

export default (
  el: React.Element<*>,
  ctx: any,
  logger?: Logger,
  enableServerStreaming?: boolean
) => {
  try {
    if (enableServerStreaming) {
      ctx.res.socket.on('error', (error) => {
        console.error('Fatal server-side streaming error', error);
      });

      // let didError = false;
      const stream = new Stream.PassThrough();
      ctx.body = stream;

      const syncUrls = getUrls(ctx, ctx.syncChunks).map(
        ({url, crossOriginAttribute}) => {
          return url;
        }
      );

      const shellTemplates = ctx.shellTemplate(ctx);

      const {pipe} = renderToPipeableStream(<div id="root">{el}</div>, {
        nonce: ctx.nonce,
        bootstrapScripts: syncUrls,
        onShellReady() {
          console.log('onShellReady called');
          ctx.response.set('Content-Type', 'txt/html');
          ctx.type = 'html';
          stream.write(shellTemplates[0]);
          pipe(stream);
          stream.write(shellTemplates[1]);
        },
        onError(x) {
          // didError = true;
          console.error(x);
        },
      });
    } else {
      return `<div id='root'>${renderToString(el)}</div>`;
    }
  } catch (e) {
    if (__DEV__) {
      console.error(
        'Server-side render failed. Falling back to client-side render',
        e
      );
    }
    logger && logger.error('SSR Failed with Error', e);
    return '<div id="root" data-fusion-render="client"></div>';
  }
};
